@@ -3,10 +3,8 @@
 # 列名: 股票代码 code, 日期 trade_date
 import pandas as pd
 import numpy as np
-import seaborn as sns
 import scipy.stats as st
 import statsmodels.api as sm 
-import seaborn, pdb
 from sklearn.covariance import LedoitWolf
 from cvxopt import matrix, solvers
 from . kutil import calc_ic
