@@ -5,11 +5,14 @@ from math import pow
 from scipy import stats, optimize
 from six import iteritems
 from sys import float_info
+from .utils import nanmean, nanstd, nanmin
 
 from .periods import ANNUALIZATION_FACTORS, APPROX_BDAYS_PER_YEAR
 from .periods import DAILY, WEEKLY, MONTHLY, QUARTERLY, YEARLY
 
+
 def _create_unary_vectorized_roll_function(function):
+
     def unary_vectorized_roll(arr, window, out=None, **kwargs):
         """
         Computes the {human_readable} measure over a rolling window.
@@ -34,11 +37,10 @@ def _create_unary_vectorized_roll_function(function):
         allocated_output = out is None
 
         if len(arr):
-            out = function(
-                rolling_window(_flatten(arr), min(len(arr), window)).T,
-                out=out,
-                **kwargs
-            )
+            out = function(rolling_window(_flatten(arr), min(len(arr),
+                                                             window)).T,
+                           out=out,
+                           **kwargs)
         else:
             out = np.empty(0, dtype='float64')
 
@@ -54,7 +56,9 @@ def _create_unary_vectorized_roll_function(function):
 
     return unary_vectorized_roll
 
+
 def _create_binary_vectorized_roll_function(function):
+
     def binary_vectorized_roll(lhs, rhs, window, out=None, **kwargs):
         """
         Computes the {human_readable} measure over a rolling window.
@@ -81,12 +85,12 @@ def _create_binary_vectorized_roll_function(function):
         allocated_output = out is None
 
         if window >= 1 and len(lhs) and len(rhs):
-            out = function(
-                rolling_window(_flatten(lhs), min(len(lhs), window)).T,
-                rolling_window(_flatten(rhs), min(len(rhs), window)).T,
-                out=out,
-                **kwargs
-            )
+            out = function(rolling_window(_flatten(lhs), min(len(lhs),
+                                                             window)).T,
+                           rolling_window(_flatten(rhs), min(len(rhs),
+                                                             window)).T,
+                           out=out,
+                           **kwargs)
         elif allocated_output:
             out = np.empty(0, dtype='float64')
         else:
@@ -106,6 +110,7 @@ def _create_binary_vectorized_roll_function(function):
 
     return binary_vectorized_roll
 
+
 def annualization_factor(period, annualization):
     """
     Return annualization factor from period entered or if a custom
@@ -135,12 +140,10 @@ def annualization_factor(period, annualization):
         try:
             factor = ANNUALIZATION_FACTORS[period]
         except KeyError:
-            raise ValueError(
-                "Period cannot be '{}'. "
-                "Can be '{}'.".format(
-                    period, "', '".join(ANNUALIZATION_FACTORS.keys())
-                )
-            )
+            raise ValueError("Period cannot be '{}'. "
+                             "Can be '{}'.".format(
+                                 period,
+                                 "', '".join(ANNUALIZATION_FACTORS.keys())))
     else:
         factor = annualization
     return factor
@@ -188,6 +191,7 @@ def simple_returns(prices):
         np.divide(out, pirces[:-1], out=out)
     return out
 
+
 def cum_returns(returns, starting_value=0, out=None):
     """
     Compute cumulative returns from simple returns.
@@ -218,38 +222,41 @@ def cum_returns(returns, starting_value=0, out=None):
     cumulative_returns : array-like
         Series of cumulative returns.
     """
-    
+
     if len(returns) < 1:
         return returns.copy()
-    
+
     nanmask = np.isnan(returns)
     if np.any(nanmask):
         returns = returns.copy()
         returns[nanmask] = 0
-        
+
     allocated_output = out is None
     if allocated_output:
         out = np.empty_like(returns)
-        
+
     ## 累计净值计算
     np.add(returns, 1, out=out)
     out.cumprod(axis=0, out=out)
-    
+
     if starting_value == 0:
         np.subtract(out, 1, out=out)
     else:
         np.multiply(out, starting_value, out=out)
-        
+
     if allocated_output:
         if returns.ndim == 1 and isinstance(returns, pd.Series):
             out = pd.Series(out, index=returns.index)
         elif isinstance(returns, pd.DataFrame):
             out = pd.DataFrame(
-                out, index=returns.index, columns=returns.columns,
+                out,
+                index=returns.index,
+                columns=returns.columns,
             )
 
     return out
 
+
 def cum_returns_final(returns, starting_value=0):
     """
     Compute total returns from simple returns.
@@ -270,10 +277,10 @@ def cum_returns_final(returns, starting_value=0):
         If input is 2-dimensional (a DataFrame or 2D numpy array), the result
         is a 1D array containing cumulative returns for each column of input.
     """
-    
+
     if len(returns) == 0:
         return np.nan
-    
+
     if isinstance(returns, pd.DataFrame):
         result = (returns + 1).prod()
     else:
@@ -286,6 +293,7 @@ def cum_returns_final(returns, starting_value=0):
 
     return result
 
+
 def aggregate_returns(returns, convert_to):
     """
     Aggregates returns by week, month, quarter or year.
@@ -302,7 +310,7 @@ def aggregate_returns(returns, convert_to):
     -------
     aggregated_returns : pd.Series
     """
-    
+
     def cumulate_returns(x):
         return cum_returns(x).iloc[-1]
 
@@ -311,16 +319,16 @@ def aggregate_returns(returns, convert_to):
     elif convert_to == MONTHLY:
         grouping = [lambda x: x.year, lambda x: x.month]
     elif convert_to == QUARTERLY:
-        grouping = [lambda x: x.year, lambda x: int(math.ceil(x.month/3.))]
+        grouping = [lambda x: x.year, lambda x: int(math.ceil(x.month / 3.))]
     elif convert_to == YEARLY:
         grouping = [lambda x: x.year]
     else:
-        raise ValueError(
-            'convert_to must be {}, {} or {}'.format(WEEKLY, MONTHLY, YEARLY)
-        )
+        raise ValueError('convert_to must be {}, {} or {}'.format(
+            WEEKLY, MONTHLY, YEARLY))
 
     return returns.groupby(grouping).apply(cumulate_returns)
-        
+
+
 def max_drawdown(returns, out=None):
     """
     Determines the maximum drawdown of a strategy.
@@ -357,7 +365,7 @@ def max_drawdown(returns, out=None):
     returns_array = np.asanyarray(returns)
 
     cumulative = np.empty(
-        (returns.shape[0] + 1,) + returns.shape[1:],
+        (returns.shape[0] + 1, ) + returns.shape[1:],
         dtype='float64',
     )
     cumulative[0] = start = 100
@@ -373,8 +381,10 @@ def max_drawdown(returns, out=None):
 
     return out
 
+
 roll_max_drawdown = _create_unary_vectorized_roll_function(max_drawdown)
 
+
 def annual_return(returns, period=DAILY, annualization=None):
     """
     Determines the mean annual growth rate of returns. This is equivilent
@@ -414,7 +424,9 @@ def annual_return(returns, period=DAILY, annualization=None):
     # Pass array to ensure index -1 looks up successfully.
     ending_value = cum_returns_final(returns, starting_value=1)
 
-    return ending_value ** (1 / num_years) - 1  # (1+r)^n = end_value -- r 年化收益  n 年数
+    return ending_value**(1 /
+                          num_years) - 1  # (1+r)^n = end_value -- r 年化收益  n 年数
+
 
 def cagr(returns, period=DAILY, annualization=None):
     """
@@ -449,6 +461,7 @@ def cagr(returns, period=DAILY, annualization=None):
     """
     return annual_return(returns, period, annualization)
 
+
 roll_cagr = _create_unary_vectorized_roll_function(cagr)
 
 
@@ -502,15 +515,14 @@ def annual_volatility(returns,
 
     ann_factor = annualization_factor(period, annualization)
     np.nanstd(returns, ddof=1, axis=0, out=out)
-    out = np.multiply(out, ann_factor ** (1.0 / alpha), out=out)
+    out = np.multiply(out, ann_factor**(1.0 / alpha), out=out)
     if returns_1d:
         out = out.item()
     return out
 
 
 roll_annual_volatility = _create_unary_vectorized_roll_function(
-    annual_volatility,
-)
+    annual_volatility, )
 
 
 def calmar_ratio(returns, period=DAILY, annualization=None):
@@ -550,11 +562,9 @@ def calmar_ratio(returns, period=DAILY, annualization=None):
 
     max_dd = max_drawdown(returns=returns)
     if max_dd < 0:
-        temp = annual_return(
-            returns=returns,
-            period=period,
-            annualization=annualization
-        ) / abs(max_dd)
+        temp = annual_return(returns=returns,
+                             period=period,
+                             annualization=annualization) / abs(max_dd)
     else:
         return np.nan
 
@@ -564,7 +574,9 @@ def calmar_ratio(returns, period=DAILY, annualization=None):
     return temp
 
 
-def omega_ratio(returns, risk_free=0.0, required_return=0.0,
+def omega_ratio(returns,
+                risk_free=0.0,
+                required_return=0.0,
                 annualization=APPROX_BDAYS_PER_YEAR):
     """Determines the Omega ratio of a strategy.
 
@@ -615,7 +627,7 @@ def omega_ratio(returns, risk_free=0.0, required_return=0.0,
         return numer / denom
     else:
         return np.nan
-    
+
 
 def sharpe_ratio(returns,
                  risk_free=0,
@@ -688,8 +700,10 @@ def sharpe_ratio(returns,
 
     return out
 
+
 roll_sharpe_ratio = _create_unary_vectorized_roll_function(sharpe_ratio)
 
+
 def sortino_ratio(returns,
                   required_return=0,
                   period=DAILY,
@@ -757,11 +771,9 @@ def sortino_ratio(returns,
     ann_factor = annualization_factor(period, annualization)
 
     average_annual_return = nanmean(adj_returns, axis=0) * ann_factor
-    annualized_downside_risk = (
-        _downside_risk
-        if _downside_risk is not None else
-        downside_risk(returns, required_return, period, annualization)
-    )
+    annualized_downside_risk = (_downside_risk if _downside_risk is not None
+                                else downside_risk(returns, required_return,
+                                                   period, annualization))
     np.divide(average_annual_return, annualized_downside_risk, out=out)
     if return_1d:
         out = out.item()
@@ -774,7 +786,6 @@ def sortino_ratio(returns,
 roll_sortino_ratio = _create_unary_vectorized_roll_function(sortino_ratio)
 
 
-
 def downside_risk(returns,
                   required_return=0,
                   period=DAILY,
@@ -858,6 +869,40 @@ def downside_risk(returns,
 roll_downsize_risk = _create_unary_vectorized_roll_function(downside_risk)
 
 
+def information_ratio(returns, factor_returns):
+    """
+    Determines the Information ratio of a strategy.
+
+    Parameters
+    ----------
+    returns : pd.Series or np.ndarray
+        Daily returns of the strategy, noncumulative.
+        - See full explanation in :func:`~empyrical.stats.cum_returns`.
+    factor_returns: float / series
+        Benchmark return to compare returns against.
+
+    Returns
+    -------
+    float
+        The information ratio.
+
+    Note
+    -----
+    See https://en.wikipedia.org/wiki/information_ratio for more details.
+
+    """
+    if len(returns) < 2:
+        return np.nan
+
+    active_return = _adjust_returns(returns, factor_returns)
+    tracking_error = nanstd(active_return, ddof=1)
+    if np.isnan(tracking_error):
+        return 0.0
+    if tracking_error == 0:
+        return np.nan
+    return nanmean(active_return) / tracking_error
+
+
 def excess_sharpe(returns, factor_returns, out=None):
     """
     Determines the Excess Sharpe of a strategy.
@@ -937,4 +982,229 @@ def tail_ratio(returns):
         return np.nan
 
     return np.abs(np.percentile(returns, 95)) / \
-        np.abs(np.percentile(returns, 5))
\ No newline at end of file
+        np.abs(np.percentile(returns, 5))
+
+
+def _aligned_series(*many_series):
+    """
+    Return a new list of series containing the data in the input series, but
+    with their indices aligned. NaNs will be filled in for missing values.
+
+    Parameters
+    ----------
+    many_series : list[pd.Series]
+
+    Returns
+    -------
+    aligned_series : list[pd.Series]
+
+        A new list of series containing the data in the input series, but
+        with their indices aligned. NaNs will be filled in for missing values.
+
+    """
+    return [
+        series for col, series in iteritems(pd.concat(many_series, axis=1))
+    ]
+
+
+def alpha_beta(returns,
+               factor_returns,
+               risk_free=0.0,
+               period=DAILY,
+               annualization=None):
+    """Calculates annualized alpha and beta.
+
+    Parameters
+    ----------
+    returns : pd.Series
+        Daily returns of the strategy, noncumulative.
+        - See full explanation in :func:`~empyrical.stats.cum_returns`.
+    factor_returns : pd.Series
+         Daily noncumulative returns of the factor to which beta is
+         computed. Usually a benchmark such as the market.
+         - This is in the same style as returns.
+    risk_free : int, float, optional
+        Constant risk-free return throughout the period. For example, the
+        interest rate on a three month us treasury bill.
+    period : str, optional
+        Defines the periodicity of the 'returns' data for purposes of
+        annualizing. Value ignored if `annualization` parameter is specified.
+        Defaults are:
+            'monthly':12
+            'weekly': 52
+            'daily': 252
+    annualization : int, optional
+        Used to suppress default values available in `period` to convert
+        returns into annual returns. Value should be the annual frequency of
+        `returns`.
+
+    Returns
+    -------
+    float
+        Alpha.
+    float
+        Beta.
+
+    """
+    if len(returns) < 2 or len(factor_returns) < 2:
+        return np.nan, np.nan
+
+    return alpha_beta_aligned(*_aligned_series(returns, factor_returns),
+                              risk_free=risk_free,
+                              period=period,
+                              annualization=annualization)
+
+
+def beta_aligned(returns, factor_returns, risk_free=0.0):
+    """Calculates beta.
+
+    If they are pd.Series, expects returns and factor_returns have already
+    been aligned on their labels.  If np.ndarray, these arguments should have
+    the same shape.
+
+    Parameters
+    ----------
+    returns : pd.Series or np.ndarray
+        Daily returns of the strategy, noncumulative.
+        - See full explanation in :func:`~empyrical.stats.cum_returns`.
+    factor_returns : pd.Series or np.ndarray
+         Daily noncumulative returns of the factor to which beta is
+         computed. Usually a benchmark such as the market.
+         - This is in the same style as returns.
+    risk_free : int, float, optional
+        Constant risk-free return throughout the period. For example, the
+        interest rate on a three month us treasury bill.
+
+    Returns
+    -------
+    float
+        Beta.
+    """
+
+    if len(returns) < 2 or len(factor_returns) < 2:
+        return np.nan
+    # Filter out dates with np.nan as a return value
+    joint = np.vstack([_adjust_returns(returns, risk_free), factor_returns])
+    joint = joint[:, ~np.isnan(joint).any(axis=0)]
+    if joint.shape[1] < 2:
+        return np.nan
+
+    cov = np.cov(joint, ddof=0)
+
+    if np.absolute(cov[1, 1]) < 1.0e-30:
+        return np.nan
+
+    return cov[0, 1] / cov[1, 1]
+
+
+def alpha_aligned(returns,
+                  factor_returns,
+                  risk_free=0.0,
+                  period=DAILY,
+                  annualization=None,
+                  _beta=None):
+    """Calculates annualized alpha.
+
+    If they are pd.Series, expects returns and factor_returns have already
+    been aligned on their labels.  If np.ndarray, these arguments should have
+    the same shape.
+
+    Parameters
+    ----------
+    returns : pd.Series or np.ndarray
+        Daily returns of the strategy, noncumulative.
+        - See full explanation in :func:`~empyrical.stats.cum_returns`.
+    factor_returns : pd.Series or np.ndarray
+         Daily noncumulative returns of the factor to which beta is
+         computed. Usually a benchmark such as the market.
+         - This is in the same style as returns.
+    risk_free : int, float, optional
+        Constant risk-free return throughout the period. For example, the
+        interest rate on a three month us treasury bill.
+    period : str, optional
+        Defines the periodicity of the 'returns' data for purposes of
+        annualizing. Value ignored if `annualization` parameter is specified.
+        Defaults are:
+            'monthly':12
+            'weekly': 52
+            'daily': 252
+    annualization : int, optional
+        Used to suppress default values available in `period` to convert
+        returns into annual returns. Value should be the annual frequency of
+        `returns`.
+        - See full explanation in :func:`~empyrical.stats.annual_return`.
+    _beta : float, optional
+        The beta for the given inputs, if already known. Will be calculated
+        internally if not provided.
+
+    Returns
+    -------
+    float
+        Alpha.
+    """
+    if len(returns) < 2:
+        return np.nan
+
+    ann_factor = annualization_factor(period, annualization)
+
+    if _beta is None:
+        _beta = beta_aligned(returns, factor_returns, risk_free)
+
+    adj_returns = _adjust_returns(returns, risk_free)
+    adj_factor_returns = _adjust_returns(factor_returns, risk_free)
+    alpha_series = adj_returns - (_beta * adj_factor_returns)
+
+    return nanmean(alpha_series) * ann_factor
+
+
+def alpha_beta_aligned(returns,
+                       factor_returns,
+                       risk_free=0.0,
+                       period=DAILY,
+                       annualization=None):
+    """Calculates annualized alpha and beta.
+
+    If they are pd.Series, expects returns and factor_returns have already
+    been aligned on their labels.  If np.ndarray, these arguments should have
+    the same shape.
+
+    Parameters
+    ----------
+    returns : pd.Series or np.ndarray
+        Daily returns of the strategy, noncumulative.
+        - See full explanation in :func:`~empyrical.stats.cum_returns`.
+    factor_returns : pd.Series or np.ndarray
+         Daily noncumulative returns of the factor to which beta is
+         computed. Usually a benchmark such as the market.
+         - This is in the same style as returns.
+    risk_free : int, float, optional
+        Constant risk-free return throughout the period. For example, the
+        interest rate on a three month us treasury bill.
+    period : str, optional
+        Defines the periodicity of the 'returns' data for purposes of
+        annualizing. Value ignored if `annualization` parameter is specified.
+        Defaults are:
+            'monthly':12
+            'weekly': 52
+            'daily': 252
+    annualization : int, optional
+        Used to suppress default values available in `period` to convert
+        returns into annual returns. Value should be the annual frequency of
+        `returns`.
+
+    Returns
+    -------
+    float
+        Alpha.
+    float
+        Beta.
+
+    """
+    b = beta_aligned(returns, factor_returns, risk_free)
+    a = alpha_aligned(returns,
+                      factor_returns,
+                      risk_free,
+                      period,
+                      annualization,
+                      _beta=b)
+    return a, b
\ No newline at end of file
