@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import numpy as np
-from .... utilities.singleton import Singleton
+from ....utilities.singleton import Singleton
 from ultron.sentry.Analysis.SecurityValueHolders import SecurityLatestValueHolder
 from ultron.sentry.Analysis.SecurityValueHolders import SecurityCurrentValueHolder
 
@@ -80,20 +80,21 @@ from ultron.sentry.Analysis.TechnicalAnalysis.StatefulTechnicalAnalysers import
 from ultron.sentry.Analysis.SecurityValueHolders import SecurityDeltaValueHolder
 from ultron.sentry.Analysis.SecurityValueHolders import SecurityShiftedValueHolder
 
-
 ## 时间序列交叉
 from ultron.sentry.Analysis.TechnicalAnalysis.StatefulTechnicalAnalysers import SecurityMovingCorrelation
 
-
-import six,pdb
+import six
 from enum import Enum, unique
 
+
 @unique
 class FunctionType(Enum):
     cross_section = 1
     time_series = 2
 
+
 class Function(object):
+
     def __init__(self, function, arity, ftype, default_value=0):
         self.function = function
         self.arity = arity
@@ -101,91 +102,115 @@ class Function(object):
         self.ftype = ftype
         self.default_value = default_value
 
+
 @six.add_metaclass(Singleton)
 class Operators(object):
+
     def __init__(self):
-       
+
         # 时间序列默认周期列表
-        self._ts_period = [2, 4, 6, 8 ,10]
-        
+        self._ts_period = [2, 4, 6, 8, 10]
+
         self._init_cs()
         self._init_ts()
-        
-        self._cs_mutation_function_list = [Function(f, 1, FunctionType.cross_section) for f in self._cross_section_mutation_list]
-        self._cs_crossover_function_list = [Function(f, 2, FunctionType.cross_section) for f in self._cross_section_crossover_list]
+
+        self._cs_mutation_function_list = [
+            Function(f, 1, FunctionType.cross_section)
+            for f in self._cross_section_mutation_list
+        ]
+        self._cs_crossover_function_list = [
+            Function(f, 2, FunctionType.cross_section)
+            for f in self._cross_section_crossover_list
+        ]
         #支持全部参数
         self._ts_mutation_function_list = []
         self._ts_crossover_function_list = []
         for period in self._ts_period:
-            self._ts_mutation_function_list += [Function(f, 1, FunctionType.time_series, period) for f in self._time_series_mutation_list]
-            self._ts_crossover_function_list += [Function(f, 2, FunctionType.time_series, period) for f in self._time_series_crossover_list]
-        
+            self._ts_mutation_function_list += [
+                Function(f, 1, FunctionType.time_series, period)
+                for f in self._time_series_mutation_list
+            ]
+            self._ts_crossover_function_list += [
+                Function(f, 2, FunctionType.time_series, period)
+                for f in self._time_series_crossover_list
+            ]
+
         self._mutation_sets = self._cs_mutation_function_list + self._ts_mutation_function_list
         self._crossover_sets = self._cs_crossover_function_list + self._ts_crossover_function_list
-        
-    
+
     def _init_cs(self):
-        self._cross_section_mutation_list = [SecurityAverageValueHolder,SecurityDiffValueHolder,
-                                             SecurityTanhValueHolder,SecuritySignValueHolder,
-                                             SecurityExpValueHolder, SecurityLogValueHolder,SecuritySqrtValueHolder,
-                                             SecurityAbsValueHolder,SecurityAcosValueHolder,
-                                             SecurityAsinValueHolder,SecurityAsinhValueHolder,
-                               SecurityAcoshValueHolder,SecurityNormInvValueHolder,
-                               SecurityCeilValueHolder,SecurityFloorValueHolder,
-                               SecurityRoundValueHolder,SecuritySimpleReturnValueHolder,
-                               SecurityLogReturnValueHolder,SecuritySigmoidValueHolder,
-                               CSRankedSecurityValueHolder,CSZScoreSecurityValueHolder,
-                               CSPercentileSecurityValueHolder,CSAverageSecurityValueHolder,
-                               CSAverageAdjustedSecurityValueHolder]
-        
-        self._cross_section_crossover_list = [CSResidueSecurityValueHolder,SecurityAddedValueHolder,
-                                          SecuritySubbedValueHolder,SecurityMultipliedValueHolder,
-                                          SecurityDividedValueHolder,SecurityLtOperatorValueHolder,
-                                          SecurityLeOperatorValueHolder,SecurityGtOperatorValueHolder,
-                                          SecurityGeOperatorValueHolder,SecurityEqOperatorValueHolder,
-                                          SecurityNeOperatorValueHolder,SecurityAndOperatorValueHolder,
-                                          SecurityOrOperatorValueHolder,SecurityMinimumValueHolder,
-                                          SecurityMaximumValueHolder]
-        
+        self._cross_section_mutation_list = [
+            SecurityAverageValueHolder, SecurityDiffValueHolder,
+            SecurityTanhValueHolder, SecuritySignValueHolder,
+            SecurityExpValueHolder, SecurityLogValueHolder,
+            SecuritySqrtValueHolder, SecurityAbsValueHolder,
+            SecurityAcosValueHolder, SecurityAsinValueHolder,
+            SecurityAsinhValueHolder, SecurityAcoshValueHolder,
+            SecurityNormInvValueHolder, SecurityCeilValueHolder,
+            SecurityFloorValueHolder, SecurityRoundValueHolder,
+            SecuritySimpleReturnValueHolder, SecurityLogReturnValueHolder,
+            SecuritySigmoidValueHolder, CSRankedSecurityValueHolder,
+            CSZScoreSecurityValueHolder, CSPercentileSecurityValueHolder,
+            CSAverageSecurityValueHolder, CSAverageAdjustedSecurityValueHolder
+        ]
+
+        self._cross_section_crossover_list = [
+            CSResidueSecurityValueHolder, SecurityAddedValueHolder,
+            SecuritySubbedValueHolder, SecurityMultipliedValueHolder,
+            SecurityDividedValueHolder, SecurityLtOperatorValueHolder,
+            SecurityLeOperatorValueHolder, SecurityGtOperatorValueHolder,
+            SecurityGeOperatorValueHolder, SecurityEqOperatorValueHolder,
+            SecurityNeOperatorValueHolder, SecurityAndOperatorValueHolder,
+            SecurityOrOperatorValueHolder, SecurityMinimumValueHolder,
+            SecurityMaximumValueHolder
+        ]
+
     def _init_ts(self):
         self._time_series_mutation_list = [
-            SecurityXAverageValueHolder,SecurityMovingAverage,
-            SecurityMovingDecay,SecurityMovingMax,
-            SecurityMovingArgMax,SecurityMovingMin,
-            SecurityMovingArgMin,SecurityMovingRank,
-            SecurityMovingQuantile,SecurityMovingAllTrue,
-            SecurityMovingAnyTrue,SecurityMovingSum,
-            SecurityMovingVariance,SecurityMovingStandardDeviation,
-            SecurityMovingCountedPositive,SecurityMovingPositiveAverage,
-            SecurityMovingPositiveDifferenceAverage,SecurityMovingNegativeDifferenceAverage,
-            SecurityMovingRSI,SecurityMovingLogReturn,SecurityDeltaValueHolder,
+            SecurityXAverageValueHolder, SecurityMovingAverage,
+            SecurityMovingDecay, SecurityMovingMax, SecurityMovingArgMax,
+            SecurityMovingMin, SecurityMovingArgMin, SecurityMovingRank,
+            SecurityMovingQuantile, SecurityMovingAllTrue,
+            SecurityMovingAnyTrue, SecurityMovingSum, SecurityMovingVariance,
+            SecurityMovingStandardDeviation, SecurityMovingCountedPositive,
+            SecurityMovingPositiveAverage,
+            SecurityMovingPositiveDifferenceAverage,
+            SecurityMovingNegativeDifferenceAverage, SecurityMovingRSI,
+            SecurityMovingLogReturn, SecurityDeltaValueHolder,
             SecurityShiftedValueHolder
         ]
-        
-        self._time_series_crossover_list = [
-            SecurityMovingCorrelation
-        ]
-        
-        
+
+        self._time_series_crossover_list = [SecurityMovingCorrelation]
+
     def custom_transformer(self, formula_sets):
-        operators_sets = self._crossover_sets + self._mutation_sets 
-        return [operator for operator in operators_sets if operator.name in formula_sets]
-        
-    def calc_factor(self, expression, total_data, indexs, key, name='transformed', dropna=False):
-        return eval(expression).transform(total_data.set_index(indexs), 
-                                         category_field=key, name=name,
-                                         dropna=dropna)
-    
+        operators_sets = self._crossover_sets + self._mutation_sets
+        return [
+            operator for operator in operators_sets
+            if operator.name in formula_sets
+        ]
+
+    def calc_factor(self,
+                    expression,
+                    total_data,
+                    indexs,
+                    key,
+                    name='transformed',
+                    dropna=False):
+        return eval(expression).transform(total_data.set_index(indexs),
+                                          category_field=key,
+                                          name=name,
+                                          dropna=dropna)
+
     def expression_params(self, expression):
         return eval(expression)._dependency
-        
+
     def fetch_mutation_sets(self):
         return self._mutation_sets
-    
+
     def fetch_crossover_sets(self):
         return self._crossover_sets
-    
-    
+
+
 mutation_sets = Operators().fetch_mutation_sets()
 crossover_sets = Operators().fetch_crossover_sets()
 operators_sets = mutation_sets + crossover_sets
