@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-import pdb
 import numpy as np
 from sklearn.linear_model import LinearRegression as LinearRegressionImpl
 from sklearn.linear_model import Lasso as LassoRegressionImpl
@@ -20,7 +19,8 @@ class ConstLinearModelImpl(object):
         self.weights = weights.flatten()
 
     def fit(self, x: np.ndarray, y: np.ndarray):
-        raise NotImplementedError("Const linear model doesn't offer fit methodology")
+        raise NotImplementedError(
+            "Const linear model doesn't offer fit methodology")
 
     def predict(self, x: np.ndarray):
         return x @ self.weights
@@ -28,24 +28,21 @@ class ConstLinearModelImpl(object):
     def score(self, x: np.ndarray, y: np.ndarray) -> float:
         y_hat = self.predict(x)
         y_bar = y.mean()
-        ssto = ((y - y_bar) ** 2).sum()
-        sse = ((y - y_hat) ** 2).sum()
+        ssto = ((y - y_bar)**2).sum()
+        sse = ((y - y_hat)**2).sum()
         return 1. - sse / ssto
 
 
 class ConstLinearModel(create_model_base()):
 
-    def __init__(self,
-                 features=None,
-                 weights: dict = None,
-                 fit_target=None):
+    def __init__(self, features=None, weights: dict = None, fit_target=None):
         super().__init__(features=features, fit_target=fit_target)
         if features is not None and weights is not None:
-            assert(len(features) == len(weights),
-                        ValueError,
-                        "length of features is not equal to length of weights")
+            assert (len(features) == len(weights), ValueError,
+                    "length of features is not equal to length of weights")
         if weights:
-            self.impl = ConstLinearModelImpl(np.array([weights[name] for name in self.features]))
+            self.impl = ConstLinearModelImpl(
+                np.array([weights[name] for name in self.features]))
 
     def save(self):
         model_desc = super().save()
@@ -63,7 +60,11 @@ class ConstLinearModel(create_model_base()):
 
 class LinearRegression(create_model_base('sklearn')):
 
-    def __init__(self, features=None, fit_intercept: bool = False, fit_target=None, **kwargs):
+    def __init__(self,
+                 features=None,
+                 fit_intercept: bool = False,
+                 fit_target=None,
+                 **kwargs):
         super().__init__(features=features, fit_target=fit_target)
         self.impl = LinearRegressionImpl(fit_intercept=fit_intercept, **kwargs)
 
@@ -79,9 +80,16 @@ class LinearRegression(create_model_base('sklearn')):
 
 class LassoRegression(create_model_base('sklearn')):
 
-    def __init__(self, alpha=0.01, features=None, fit_intercept: bool = False, fit_target=None, **kwargs):
+    def __init__(self,
+                 alpha=0.01,
+                 features=None,
+                 fit_intercept: bool = False,
+                 fit_target=None,
+                 **kwargs):
         super().__init__(features=features, fit_target=fit_target)
-        self.impl = LassoRegressionImpl(alpha=alpha, fit_intercept=fit_intercept, **kwargs)
+        self.impl = LassoRegressionImpl(alpha=alpha,
+                                        fit_intercept=fit_intercept,
+                                        **kwargs)
 
     def save(self) -> dict:
         model_desc = super().save()
@@ -92,39 +100,66 @@ class LassoRegression(create_model_base('sklearn')):
     def weights(self):
         return self.impl.coef_.tolist()
 
+
 class BayesianRegression(create_model_base('sklearn')):
-    def __init__(self, alpha=0.01, features=None, fit_intercept: bool = False, fit_target=None, **kwargs):
+
+    def __init__(self,
+                 alpha=0.01,
+                 features=None,
+                 fit_intercept: bool = False,
+                 fit_target=None,
+                 **kwargs):
         super().__init__(features=features, fit_target=fit_target)
-        self.impl = BayesianRidgeImpl(alpha_1=alpha, fit_intercept=fit_intercept, **kwargs)
-    
+        self.impl = BayesianRidgeImpl(alpha_1=alpha,
+                                      fit_intercept=fit_intercept,
+                                      **kwargs)
+
     def save(self) -> dict:
         model_desc = super().save()
         model_desc['weight'] = self.impl.coef_.tolist()
         return model_desc
-    
+
     @property
     def weights(self):
         return self.impl.coef_.tolist()
 
+
 class RidgeRegression(create_model_base('sklearn')):
-    def __init__(self, alpha=0.01, features=None, fit_intercept: bool = False, fit_target=None, **kwargs):
+
+    def __init__(self,
+                 alpha=0.01,
+                 features=None,
+                 fit_intercept: bool = False,
+                 fit_target=None,
+                 **kwargs):
         super().__init__(features=features, fit_target=fit_target)
-        self.impl = RidgeImpl(alpha=alpha, fit_intercept=fit_intercept, **kwargs)
-    
+        self.impl = RidgeImpl(alpha=alpha,
+                              fit_intercept=fit_intercept,
+                              **kwargs)
+
     def save(self) -> dict:
         model_desc = super().save()
         model_desc['weight'] = self.impl.coef_.tolist()
         return model_desc
-    
+
     @property
     def weights(self):
         return self.impl.coef_.tolist()
 
+
 class TweedieRegression(create_model_base('sklearn')):
-    def __init__(self, alpha=0.01, features=None, fit_intercept: bool = False, fit_target=None, **kwargs):
+
+    def __init__(self,
+                 alpha=0.01,
+                 features=None,
+                 fit_intercept: bool = False,
+                 fit_target=None,
+                 **kwargs):
         super().__init__(features=features, fit_target=fit_target)
-        self.impl = TweedieRegressorImpl(alpha=alpha, fit_intercept=fit_intercept, **kwargs)
-    
+        self.impl = TweedieRegressorImpl(alpha=alpha,
+                                         fit_intercept=fit_intercept,
+                                         **kwargs)
+
     def save(self) -> dict:
         model_desc = super().save()
         model_desc['weight'] = self.impl.coef_.tolist()
@@ -134,11 +169,20 @@ class TweedieRegression(create_model_base('sklearn')):
     def weights(self):
         return self.impl.coef_.tolist()
 
+
 class HuberRegression(create_model_base('sklearn')):
-    def __init__(self, alpha=0.01, features=None, fit_intercept: bool = False, fit_target=None, **kwargs):
+
+    def __init__(self,
+                 alpha=0.01,
+                 features=None,
+                 fit_intercept: bool = False,
+                 fit_target=None,
+                 **kwargs):
         super().__init__(features=features, fit_target=fit_target)
-        self.impl = HuberRegressorImpl(alpha=alpha, fit_intercept=fit_intercept, **kwargs)
-    
+        self.impl = HuberRegressorImpl(alpha=alpha,
+                                       fit_intercept=fit_intercept,
+                                       **kwargs)
+
     def save(self) -> dict:
         model_desc = super().save()
         model_desc['weight'] = self.impl.coef_.tolist()
@@ -148,10 +192,19 @@ class HuberRegression(create_model_base('sklearn')):
     def weights(self):
         return self.impl.coef_.tolist()
 
+
 class SGDRegression(create_model_base('sklearn')):
-    def __init__(self, alpha=0.001, features=None, fit_intercept: bool = False, fit_target=None, **kwargs):
+
+    def __init__(self,
+                 alpha=0.001,
+                 features=None,
+                 fit_intercept: bool = False,
+                 fit_target=None,
+                 **kwargs):
         super().__init__(features=features, fit_target=fit_target)
-        self.impl = SGDRegressorImpl(alpha=alpha, fit_intercept=fit_intercept, **kwargs)
+        self.impl = SGDRegressorImpl(alpha=alpha,
+                                     fit_intercept=fit_intercept,
+                                     **kwargs)
 
     def save(self) -> dict:
         model_desc = super().save()
@@ -162,10 +215,17 @@ class SGDRegression(create_model_base('sklearn')):
     def weights(self):
         return self.impl.coef_.tolist()
 
+
 class PassiveAggressiveRegression(create_model_base('sklearn')):
-    def __init__(self, features=None, fit_intercept: bool = False, fit_target=None, **kwargs):
+
+    def __init__(self,
+                 features=None,
+                 fit_intercept: bool = False,
+                 fit_target=None,
+                 **kwargs):
         super().__init__(features=features, fit_target=fit_target)
-        self.impl = PassiveAggressiveRegressorImpl(fit_intercept=fit_intercept, **kwargs)
+        self.impl = PassiveAggressiveRegressorImpl(fit_intercept=fit_intercept,
+                                                   **kwargs)
 
     def save(self) -> dict:
         model_desc = super().save()
@@ -176,10 +236,17 @@ class PassiveAggressiveRegression(create_model_base('sklearn')):
     def weights(self):
         return self.impl.coef_.tolist()
 
+
 class TheilSenRegression(create_model_base('sklearn')):
-    def __init__(self, features=None, fit_intercept: bool = False, fit_target=None, **kwargs):
+
+    def __init__(self,
+                 features=None,
+                 fit_intercept: bool = False,
+                 fit_target=None,
+                 **kwargs):
         super().__init__(features=features, fit_target=fit_target)
-        self.impl = TheilSenRegressorImpl(fit_intercept=fit_intercept, **kwargs)
+        self.impl = TheilSenRegressorImpl(fit_intercept=fit_intercept,
+                                          **kwargs)
 
     def save(self) -> dict:
         model_desc = super().save()
@@ -190,11 +257,17 @@ class TheilSenRegression(create_model_base('sklearn')):
     def weights(self):
         return self.impl.coef_.tolist()
 
+
 class LogisticRegression(create_model_base('sklearn')):
 
-    def __init__(self, features=None, fit_intercept: bool = False, fit_target=None, **kwargs):
+    def __init__(self,
+                 features=None,
+                 fit_intercept: bool = False,
+                 fit_target=None,
+                 **kwargs):
         super().__init__(features=features, fit_target=fit_target)
-        self.impl = LogisticRegressionImpl(fit_intercept=fit_intercept, **kwargs)
+        self.impl = LogisticRegressionImpl(fit_intercept=fit_intercept,
+                                           **kwargs)
 
     def save(self) -> dict:
         model_desc = super().save()
@@ -203,4 +276,4 @@ class LogisticRegression(create_model_base('sklearn')):
 
     @property
     def weights(self):
-        return self.impl.coef_.tolist()
\ No newline at end of file
+        return self.impl.coef_.tolist()
